﻿/**********************************************************************\
**
**
**                          Copyright (c) 2015-2016 by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a license and may be used and
**      copied only in accordance with the terms  of  such  license  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Xpm-ApiWrapperDotNetDemo-MainForm.cs
**  Facility:    S690001 - Hengstler XPM Printer Driver for Windows XP/7/8/10
**  Component:   Xpm ApiWrapper .NET Demo Application
**  Version:     R3-V1.02-FINAL / 2016AUG24
**  Environment: Windows XP/7/8/10 / .NET Framework V4.0
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Handles main form
**  Note:        -
**  Note:        -
**  History:     R2-V1.01e / 2014DEC05 / HOR / M-R2-035-HOR
**                  Application added which demonstrates how to interface the XPM .NET Wrapper DLL.
**               R2-V1.01e / 2015MAY05 / HOR / 2nd RELEASE - 5th BETA
**               R2-V1.01f / 2016FEB15 / HOR / 2nd RELEASE - 6th BETA
**               R2-V1.01  / 2016AUG19 / HOR / M-R2-041-HOR
**                 Complete driver package converted to VS2015 and adapted to Windows 8/10 32/64 bit
**               R2-V1.01  / 2016AUG19 / HOR / 2nd RELEASE - FINAL
**               ------------------------------------------------------------
**               R3-V1.02  / 2016AUG24 / HOR / M-R3-001-HOR
**                 Additional driver package compilation added for usage within Windows-XP.
**               R3-V1.02  / 2016AUG24 / HOR / 3rd RELEASE - FINAL
**               ------------------------------------------------------------
**
\**********************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Xpm_ApiWrapperDotNet;
using System.Runtime.InteropServices;
using System.Threading;

namespace Xpm_ApiWrapperDotNetDemo
{
    public partial class MainForm : Form
    {
        /*
         * CONSTANTS
         */        
        
        /*
         * MEMBER ATTRIBUTES
         */
        
        Xpm_ApiWrapper   m_xpmApi            = null;
        
        /*
         * METHODS
         */

        // -- Default Constructor
        public MainForm()
        {
            InitializeComponent();
        }
        
        // -- Method executed when form gets visible
        private void MainForm_Shown(object sender, EventArgs e)
        {
            // -- Try to init XPM C# API 
            // .. Load library Xpm_Api.dll
            try
            {
                //-- Create new XPM API Wrapper
                m_xpmApi = new Xpm_ApiWrapper();

                // -- Enumerate all XPM printer and output names to combo box
                IntPtr hPrintEnum;
                if ((hPrintEnum = m_xpmApi.Xpm_Api_Pe_PrinterEnumStart()) != IntPtr.Zero)
                {
                    //-- Add all XPM printers
                    string strPrinterName = string.Empty;
                    for (strPrinterName = m_xpmApi.Xpm_Api_Pe_PrinterEnum(hPrintEnum, true);
                        strPrinterName != null;
                        strPrinterName = m_xpmApi.Xpm_Api_Pe_PrinterEnum(hPrintEnum, false))
                    {
                        m_cmbPrinterList.Items.Add(strPrinterName);
                    }

                    //-- Select XPM printer
                    if (m_cmbPrinterList.Items.Count > 0)
                        m_cmbPrinterList.SelectedIndex = 0;

                    //-- End XPM print4er enumeration 
                    m_xpmApi.Xpm_Api_Pe_PrinterEnumEnd(hPrintEnum);
                }
            }
            catch (Xpm_ApiException ex)
            {
                MessageBox.Show(ex.Message + " APPLICATION CANCELLED", "XPM API WRAPPER FAULT", MessageBoxButtons.OK, MessageBoxIcon.Error);

                // -- Close form on error
                this.Close();
            }
                        
        }

        // -- Method executed when Form is closed
        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
        }
                
        // -- Method fo connect to selected printer
        private IntPtr connectPrinter()
        {
            IntPtr printerHandle = IntPtr.Zero; // Printer handle

            // -- Open selected printer if valid printer name
            if (m_xpmApi.Xpm_Api_Pe_ValidatePrinterName(m_cmbPrinterList.SelectedItem.ToString()))
            {
                printerHandle = m_xpmApi.Xpm_Api_Pio_PrinterOpen(m_cmbPrinterList.SelectedItem.ToString(), Xpm_ApiWrapper.D_XPM_API_PIO_OPT_NONE, 5000);
            }

            // -- Return handle of printer
            return printerHandle;
        }

        // -- Method for disconnecting from printer
        private void disconnectPrinter(IntPtr printerHandle)
        {
            // -- Close connection if  printer handle is non zero
            if (printerHandle != IntPtr.Zero)
            {
                // -- Close printer connection
                m_xpmApi.Xpm_Api_Pio_PrinterClose(printerHandle);
            }
        }

        // -- Button clicked method for inquiring printer status and output it to the printer data  text box
        private void m_btnInquirePrinterStatus_Click(object sender, EventArgs e)
        {
            String printerData   = "<NO DATA>"; // String containing printer data
          
            // -- Connect to printer 
            IntPtr printerHandle = IntPtr.Zero; // Printer handle
            if ((printerHandle = connectPrinter()) != IntPtr.Zero)
            {
                // -- Clear data reception buffer buffer
                m_xpmApi.Xpm_Api_Pio_PrinterReadClear(printerHandle);

                // -- Request printer status from printer
                ushort packetId       = Xpm_ApiWrapper.D_XPM_API_RID_RX_PRINTER_STATUS;
                byte[] printerCommand = { 0x1d, 0x61, (byte) (packetId >> 8), (byte) (packetId & 0xFF)};
                if (m_xpmApi.Xpm_Api_Pio_PrinterWrite(printerHandle, printerCommand, (uint)printerCommand.Length, 100) == printerCommand.Length)
                {
                    // -- Process data transmitted by printer 
                    Xpm_ApiWrapper.TS_XPM_API_PCKT_HEADER         packetHeader        = new Xpm_ApiWrapper.TS_XPM_API_PCKT_HEADER();
                    Xpm_ApiWrapper.TS_XPM_API_PCKT_PRINTER_STATUS packetPrinterStatus = new Xpm_ApiWrapper.TS_XPM_API_PCKT_PRINTER_STATUS();
                    if (m_xpmApi.Xpm_Api_Pio_PrinterRead(printerHandle, packetHeader, packetPrinterStatus, 500) < 0)
                    {
                        // -- No data received
                        printerData = String.Format("Printer data reception fault: No data received");
                    }
                    else
                    // .. Validate packet identifier 
                    if (packetHeader.ubId != Xpm_ApiWrapper.D_XPM_API_PID_TX_PRINTER_STATUS)
                    {
                        // -- Invalid packet identifier
                        printerData = String.Format( "Printer data reception fault: Invalid packet identifier ({0} instead of {1})"
                                                   , packetHeader.ubId
                                                   , Xpm_ApiWrapper.D_XPM_API_PID_TX_PRINTER_STATUS);
                    }
                    else
                    // .. Verify packet length
                    if (packetHeader.ubLength != Marshal.SizeOf(packetPrinterStatus))
                    {
                        // -- Invalid packet length
                        printerData = String.Format( "Data reception fault: Packet length mismatch ({0} instead of {1})"
                                                   , packetHeader.ubLength
                                                   , Marshal.SizeOf(packetPrinterStatus));
                    }   
                    // .. Process printer data
                    else
                    {
                        //-- Process printer data
                        printerData = String.Format( "packetPrinterStatus.dwStatus                    = {0}\r\n"
                                                   + "packetPrinterStatus.ubPrintJobId                = {1}\r\n"
                                                   + "packetPrinterStatus.siPrintHeadTempC            = {2}\r\n"
                                                   + "packetPrinterStatus.wBoardVoltageTv             = {3}\r\n"
                                                   + "packetPrinterStatus.ubState                     = {4}\r\n"
                                                   + "packetPrinterStatus.wErrorCode                  = {5}\r\n"
                                                   + "packetPrinterStatus.wErrorInfo                  = {6}\r\n"
                                                   , "[" + packetPrinterStatus.dwStatus.ToString("X04") + "]"
                                                   , packetPrinterStatus.ubPrintJobId.ToString()
                                                   , packetPrinterStatus.siPrintHeadTempC.ToString("+00;-00") + " °C"
                                                   , ((double)packetPrinterStatus.wBoardVoltageTv / 10).ToString("00.0") + " V"
                                                   , packetPrinterStatus.ubState.ToString()
                                                   , "[" + packetPrinterStatus.wErrorCode.ToString("X04") + "]"
                                                   , "[" + packetPrinterStatus.wErrorInfo.ToString("X04") + "]"
                                                   );
                    }
                }

                //-- Disconnect printer
                disconnectPrinter(printerHandle);
            }

            //-- Output printer data to text box
            m_txtPrinterData.Text = printerData;
        }

        // -- Button clicked method for inquiring printer info and output it to the printer data text box
        private void m_btnInquirePrinterInfo_Click(object sender, EventArgs e)
        {
            String printerData   = "<NO DATA>"; // String containing printer data
          
            // -- Connect to printer 
            IntPtr printerHandle = IntPtr.Zero; // Printer handle
            if ((printerHandle = connectPrinter()) != IntPtr.Zero)
            {
                // -- Clear data reception buffer buffer
                m_xpmApi.Xpm_Api_Pio_PrinterReadClear(printerHandle);

                // -- Request printer info from printer
                ushort packetId       = Xpm_ApiWrapper.D_XPM_API_RID_RX_PRINTER_INFO;
                byte[] printerCommand = { 0x1d, 0x61, (byte) (packetId >> 8), (byte) (packetId & 0xFF)};
                if (m_xpmApi.Xpm_Api_Pio_PrinterWrite(printerHandle, printerCommand, (uint)printerCommand.Length, 100) == printerCommand.Length)
                {
                    // -- Process data transmitted by printer 
                    Xpm_ApiWrapper.TS_XPM_API_PCKT_HEADER       packetHeader        = new Xpm_ApiWrapper.TS_XPM_API_PCKT_HEADER();
                    Xpm_ApiWrapper.TS_XPM_API_PCKT_PRINTER_INFO packetPrinterInfo   = new Xpm_ApiWrapper.TS_XPM_API_PCKT_PRINTER_INFO();
                    if (m_xpmApi.Xpm_Api_Pio_PrinterRead(printerHandle, packetHeader, packetPrinterInfo, 500) < 0)
                    {
                        // -- No data received
                        printerData = String.Format("Printer data reception fault: No data received");
                    }
                    else
                    // .. Validate packet identifier 
                    if (packetHeader.ubId != Xpm_ApiWrapper.D_XPM_API_PID_TX_PRINTER_INFO)
                    {
                        // -- Invalid packet identifier
                        printerData = String.Format( "Printer data reception fault: Invalid packet identifier ({0} instead of {1})"
                                                   , packetHeader.ubId
                                                   , Xpm_ApiWrapper.D_XPM_API_PID_TX_PRINTER_INFO);
                    }
                    else
                    // .. Verify packet length
                    if (packetHeader.ubLength != Marshal.SizeOf(packetPrinterInfo))
                    {
                        // -- Invalid packet length
                        printerData = String.Format( "Data reception fault: Packet length mismatch ({0} instead of {1})"
                                                   , packetHeader.ubLength
                                                   , Marshal.SizeOf(packetPrinterInfo));
                    }   
                    // .. Process printer data
                    else
                    {    
                        //-- Process printer data
                        printerData = String.Format( "packetPrinterInfo.ubProtocolRevision            = {0}\r\n"
                                                   + "packetPrinterInfo.szPartNr                      = {1}\r\n"
                                                   + "packetPrinterInfo.szSerialNr                    = {2}\r\n"
                                                   + "packetPrinterInfo.szDateOfManuf                 = {3}\r\n"
                                                   + "packetPrinterInfo.dwModelCode                   = {4}\r\n"
                                                   + "packetPrinterInfo.dwConfigCode                  = {5}\r\n"
                                                   + "packetPrinterInfo.dwSensorCode                  = {6}\r\n"
                                                   + "packetPrinterInfo.dwFeatureCode                 = {7}\r\n"
                                                   + "packetPrinterInfo.szFwPartNr                    = {8}\r\n"
                                                   + "packetPrinterInfo.szFwVersion                   = {9}\r\n"
                                                   + "packetPrinterInfo.szFwDate                      = {10}\r\n"
                                                   + "packetPrinterInfo.dwFwPatch                     = {11}\r\n"
                                                   , packetPrinterInfo.ubProtocolRevision.ToString()
                                                   , new String(packetPrinterInfo.szPartNr,      0, Array.FindIndex(packetPrinterInfo.szPartNr,      item => item == 0))               
                                                   , new String(packetPrinterInfo.szSerialNr,    0, Array.FindIndex(packetPrinterInfo.szSerialNr,    item => item == 0))               
                                                   , new String(packetPrinterInfo.szDateOfManuf, 0, Array.FindIndex(packetPrinterInfo.szDateOfManuf, item => item == 0))               
                                                   , "[" + packetPrinterInfo.dwModelCode.ToString("X08") + "]"
                                                   , "[" + packetPrinterInfo.dwConfigCode.ToString("X08") + "]"
                                                   , "[" + packetPrinterInfo.dwSensorCode.ToString("X08") + "]"
                                                   , "[" + packetPrinterInfo.dwFeatureCode.ToString("X08") + "]"
                                                   , new String(packetPrinterInfo.szFwPartNr,   0, Array.FindIndex(packetPrinterInfo.szFwPartNr,  item => item == 0))               
                                                   , new String(packetPrinterInfo.szFwVersion,  0, Array.FindIndex(packetPrinterInfo.szFwVersion, item => item == 0))               
                                                   , new String(packetPrinterInfo.szFwDate,     0, Array.FindIndex(packetPrinterInfo.szFwDate,    item => item == 0))               
                                                   , "[" + packetPrinterInfo.dwFwPatch.ToString("X08") + "]"
                                                   );
                    }
                }

                //-- DisconnectPrinter the printer
                disconnectPrinter(printerHandle);
            }

            //-- Output printer data to text box
            m_txtPrinterData.Text = printerData;
        }

        // -- Button clicked method for inquiring sensor info and output it to the printer data text box
        private void m_btnInquirePrinterSensors_Click(object sender, EventArgs e)
        {
            String printerData   = "<NO DATA>"; // String containing printer data
          
            // -- Connect to printer 
            IntPtr printerHandle = IntPtr.Zero; // Printer handle
            if ((printerHandle = connectPrinter()) != IntPtr.Zero)
            {
                // -- Clear data reception buffer buffer
                m_xpmApi.Xpm_Api_Pio_PrinterReadClear(printerHandle);

                // -- Request printer status from printer
                ushort packetId = Xpm_ApiWrapper.D_XPM_API_RID_RX_PRINTER_SENSORS;
                byte[] printerCommand = { 0x1d, 0x61, (byte) (packetId >> 8), (byte) (packetId & 0xFF)};
                if (m_xpmApi.Xpm_Api_Pio_PrinterWrite(printerHandle, printerCommand, (uint)printerCommand.Length, 100) == printerCommand.Length)
                {
                    // -- Process data transmitted by printer 
                    Xpm_ApiWrapper.TS_XPM_API_PCKT_HEADER         packetHeader        = new Xpm_ApiWrapper.TS_XPM_API_PCKT_HEADER();
                    Xpm_ApiWrapper.TS_XPM_API_PCKT_PRINTER_SENSOR packetPrinterSensor = new Xpm_ApiWrapper.TS_XPM_API_PCKT_PRINTER_SENSOR();
                    if (m_xpmApi.Xpm_Api_Pio_PrinterRead(printerHandle, packetHeader, packetPrinterSensor, 500) < 0)
                    {
                        // -- No data received
                        printerData = String.Format("Printer data reception fault: No data received");
                    }
                    else
                    // .. Validate packet identifier 
                    if (packetHeader.ubId != Xpm_ApiWrapper.D_XPM_API_PID_TX_PRINTER_SENSOR)
                    {
                        // -- Invalid packet identifier
                        printerData = String.Format( "Printer data reception fault: Invalid packet identifier ({0} instead of {1})"
                                                   , packetHeader.ubId
                                                   , Xpm_ApiWrapper.D_XPM_API_PID_TX_PRINTER_SENSOR);
                    }
                    else
                    // .. Verify packet length
                    if (packetHeader.ubLength != Marshal.SizeOf(packetPrinterSensor))
                    {
                        // -- Invalid packet length
                        printerData = String.Format( "Data reception fault: Packet length mismatch ({0} instead of {1})"
                                                   , packetHeader.ubLength
                                                   , Marshal.SizeOf(packetPrinterSensor));
                    }   
                    // .. Process printer data
                    else
                    {    
                        //-- Process printer data
                        printerData = String.Format( "packetPrinterSensor.ubProtocolRevision          = {0}\r\n"
                                                   + "packetPrinterSensor.ubAdcMeasurement            = {1}\r\n"
                                                   + "packetPrinterSensor.wDigitalInput               = {2}\r\n"
                                                   + "packetPrinterSensor.siPrintHeadTempC            = {3}\r\n"
                                                   + "packetPrinterSensor.wBoardVoltageTv             = {4}\r\n"
                                                   + "packetPrinterSensor.ubPaperPreEndSensorAState   = {5}\r\n"
                                                   + "packetPrinterSensor.ubPaperPreEndSensorBState   = {6}\r\n"
                                                   + "packetPrinterSensor.ubPaperPathKeyState         = {7}\r\n"
                                                   + "packetPrinterSensor.ubPaperPathEntrySensorState = {8}\r\n"
                                                   + "packetPrinterSensor.ubPaperPathExitSensorState  = {9}\r\n"
                                                   + "packetPrinterSensor.ubPaperEntrySensor1State    = {10}\r\n"
                                                   + "packetPrinterSensor.ubPaperEntrySensor2State    = {11}\r\n"
                                                   + "packetPrinterSensor.ubHeadUpSensorState         = {12}\r\n"
                                                   + "packetPrinterSensor.ubCutterSensor1State        = {13}\r\n"
                                                   + "packetPrinterSensor.ubCutterSensor2State        = {14}\r\n"
                                                   + "packetPrinterSensor.ubPaperExitSensorState      = {15}\r\n"
                                                   + "packetPrinterSensor.ubAuxiliarySensorState      = {16}\r\n"
                                                   + "packetPrinterSensor.ubPresenterSensor1State     = {17}\r\n"
                                                   + "packetPrinterSensor.ubPresenterSensor2State     = {18}\r\n"
                                                   + "packetPrinterSensor.ubPresenterSensor3State     = {19}\r\n"
                                                   , packetPrinterSensor.ubProtocolRevision.ToString()
                                                   , "[" + BitConverter.ToString(packetPrinterSensor.ubAdcMeasurement).Replace("-", "]/[") + "]"
                                                   , "[" + packetPrinterSensor.wDigitalInput.ToString("X04")  + "]"
                                                   , packetPrinterSensor.siPrintHeadTempC.ToString("+00;-00") + " °C"
                                                   , ((double)packetPrinterSensor.wBoardVoltageTv / 10).ToString("00.0") + " V"
                                                   , "[" + packetPrinterSensor.ubPaperPreEndSensorAState[0].ToString("X02") + "]/"
                                                   + "[" + packetPrinterSensor.ubPaperPreEndSensorAState[1].ToString("X02") + "]/"
                                                   + "[" + packetPrinterSensor.ubPaperPreEndSensorAState[2].ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubPaperPreEndSensorBState[0].ToString("X02") + "]/"
                                                   + "[" + packetPrinterSensor.ubPaperPreEndSensorBState[1].ToString("X02") + "]/"
                                                   + "[" + packetPrinterSensor.ubPaperPreEndSensorBState[2].ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubPaperPathKeyState[0].ToString("X02") + "]/"
                                                   + "[" + packetPrinterSensor.ubPaperPathKeyState[1].ToString("X02") + "]/"
                                                   + "[" + packetPrinterSensor.ubPaperPathKeyState[2].ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubPaperPathEntrySensorState[0].ToString("X02") + "]/"
                                                   + "[" + packetPrinterSensor.ubPaperPathEntrySensorState[1].ToString("X02") + "]/"
                                                   + "[" + packetPrinterSensor.ubPaperPathEntrySensorState[2].ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubPaperPathExitSensorState[0].ToString("X02") + "]/"
                                                   + "[" + packetPrinterSensor.ubPaperPathExitSensorState[1].ToString("X02") + "]/"
                                                   + "[" + packetPrinterSensor.ubPaperPathExitSensorState[2].ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubPaperEntrySensor1State.ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubPaperEntrySensor2State.ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubHeadUpSensorState.ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubCutterSensor1State.ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubCutterSensor2State.ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubPaperExitSensorState.ToString("X02")  + "]"
                                                   , "[" + packetPrinterSensor.ubAuxiliarySensorState.ToString("X02")  + "]"
                                                   , "[" + packetPrinterSensor.ubPresenterSensor1State.ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubPresenterSensor2State.ToString("X02") + "]"
                                                   , "[" + packetPrinterSensor.ubPresenterSensor3State.ToString("X02") + "]"
                                                   );
                    }
                }

                //-- DisconnectPrinter the printer
                disconnectPrinter(printerHandle);
            }

            //-- Output printer data to text box
            m_txtPrinterData.Text = printerData;

        }

        // -- Button clicked method for inquiring printer flash and output it to the printer data text box
        private void m_btnInquirePrinterFlash_Click(object sender, EventArgs e)
        {
            String printerData   = "<NO DATA>"; // String containing printer data
          
            // -- Connect to printer 
            IntPtr printerHandle = IntPtr.Zero; // Printer handle
            if ((printerHandle = connectPrinter()) != IntPtr.Zero)
            {
                // -- Clear data reception buffer buffer
                m_xpmApi.Xpm_Api_Pio_PrinterReadClear(printerHandle);

                // -- Request printer flash from printer
                ushort packetId       = Xpm_ApiWrapper.D_XPM_API_RID_RX_PRINTER_FLASH;
                byte[] printerCommand = { 0x1d, 0x61, (byte) (packetId >> 8), (byte) (packetId & 0xFF)};
                if (m_xpmApi.Xpm_Api_Pio_PrinterWrite(printerHandle, printerCommand, (uint)printerCommand.Length, 100) == printerCommand.Length)
                {
                    // -- Process data transmitted by printer 
                    Xpm_ApiWrapper.TS_XPM_API_PCKT_HEADER        packetHeader        = new Xpm_ApiWrapper.TS_XPM_API_PCKT_HEADER();
                    Xpm_ApiWrapper.TS_XPM_API_PCKT_PRINTER_FLASH packetPrinterFlash  = new Xpm_ApiWrapper.TS_XPM_API_PCKT_PRINTER_FLASH();
                    if (m_xpmApi.Xpm_Api_Pio_PrinterRead(printerHandle, packetHeader, packetPrinterFlash, 500) < 0)
                    {
                        // -- No data received
                        printerData = String.Format("Printer data reception fault: No data received");
                    }
                    else
                    // .. Validate packet identifier 
                    if (packetHeader.ubId != Xpm_ApiWrapper.D_XPM_API_PID_TX_PRINTER_FLASH)
                    {
                        // -- Invalid packet identifier
                        printerData = String.Format( "Printer data reception fault: Invalid packet identifier ({0} instead of {1})"
                                                   , packetHeader.ubId
                                                   , Xpm_ApiWrapper.D_XPM_API_PID_TX_PRINTER_FLASH);
                    }
                    else
                    // .. Verify packet length
                    if (packetHeader.ubLength != Marshal.SizeOf(packetPrinterFlash))
                    {
                        // -- Invalid packet length
                        printerData = String.Format( "Data reception fault: Packet length mismatch ({0} instead of {1})"
                                                   , packetHeader.ubLength
                                                   , Marshal.SizeOf(packetPrinterFlash));
                    }   
                    // .. Process printer data
                    else
                    {    
                        //-- Process printer data
                        printerData = String.Format( "packetPrinterFlash.ubProtocolRevision           = {0}\r\n"
                                                   , packetPrinterFlash.ubProtocolRevision.ToString()
                                                   );
                        for (int entry = 0; entry < 6; entry++) { 
                            printerData += String.Format( "packetPrinterFlash.szInfo[{0}]                    = {1}\r\n"
                                                        , entry
                                                        , new String(packetPrinterFlash.szInfo, entry * 21, Array.FindIndex<char>(packetPrinterFlash.szInfo, entry * 21, item => item == 0) - entry * 21)   
                                                        );
                            printerData += String.Format( "packetPrinterFlash.wCrc16[{0}]                    = {1}/{2}\r\n"
                                                        , entry
                                                        , "[" + packetPrinterFlash.wCrc16[entry * 2 + 0].ToString("X04") + "]"
                                                        , "[" + packetPrinterFlash.wCrc16[entry * 2 + 1].ToString("X04") + "]"
                                                        );
                            printerData += String.Format( "packetPrinterFlash.ubState[{0}]                   = {1}\r\n"
                                                        , entry
                                                        , "[" + packetPrinterFlash.ubState[entry].ToString("X02") + "]"
                                                        );
                        }
                    }
                }

                //-- DisconnectPrinter the printer
                disconnectPrinter(printerHandle);
            }

            //-- Output printer data to text box
            m_txtPrinterData.Text = printerData;
        }

        // -- Button clicked method for inquiring printer feed and output it to the printer data text box
        private void m_btnInquirePrinterFeed_Click(object sender, EventArgs e)
        {
            String printerData   = "<NO DATA>"; // String containing printer data
          
            // -- Connect to printer 
            IntPtr printerHandle = IntPtr.Zero; // Printer handle
            if ((printerHandle = connectPrinter()) != IntPtr.Zero)
            {
                // -- Clear data reception buffer buffer
                m_xpmApi.Xpm_Api_Pio_PrinterReadClear(printerHandle);

                // -- Request printer feed from printer
                ushort packetId       = Xpm_ApiWrapper.D_XPM_API_RID_RX_PRINTER_FEED;
                byte[] printerCommand = { 0x1d, 0x61, (byte) (packetId >> 8), (byte) (packetId & 0xFF)};
                if (m_xpmApi.Xpm_Api_Pio_PrinterWrite(printerHandle, printerCommand, (uint)printerCommand.Length, 100) == printerCommand.Length)
                {
                    // -- Process data transmitted by printer 
                    Xpm_ApiWrapper.TS_XPM_API_PCKT_HEADER        packetHeader       = new Xpm_ApiWrapper.TS_XPM_API_PCKT_HEADER();
                    Xpm_ApiWrapper.TS_XPM_API_PCKT_PRINTER_FEED  packetPrinterFeed  = new Xpm_ApiWrapper.TS_XPM_API_PCKT_PRINTER_FEED();
                    if (m_xpmApi.Xpm_Api_Pio_PrinterRead(printerHandle, packetHeader, packetPrinterFeed, 500) < 0)
                    {
                        // -- No data received
                        printerData = String.Format("Printer data reception fault: No data received");
                    }
                    else
                    // .. Validate packet identifier 
                    if (packetHeader.ubId != Xpm_ApiWrapper.D_XPM_API_PID_TX_PRINTER_FEED)
                    {
                        // -- Invalid packet identifier
                        printerData = String.Format( "Printer data reception fault: Invalid packet identifier ({0} instead of {1})"
                                                   , packetHeader.ubId
                                                   , Xpm_ApiWrapper.D_XPM_API_PID_TX_PRINTER_FEED);
                    }
                    else
                    // .. Verify packet length
                    if (packetHeader.ubLength != Marshal.SizeOf(packetPrinterFeed))
                    {
                        // -- Invalid packet length
                        printerData = String.Format( "Data reception fault: Packet length mismatch ({0} instead of {1})"
                                                   , packetHeader.ubLength
                                                   , Marshal.SizeOf(packetPrinterFeed));
                    }   
                    // .. Process printer data
                    else
                    {    
                        //-- Process printer data
                        printerData = String.Format( "packetPrinterFeed.ubProtocolRevision            = {0}\r\n"
                                                   + "packetPrinterFeed.ubState                       = {1}\r\n"
                                                   + "packetPrinterFeed.ubNrOfPaperPaths              = {2}\r\n"
                                                   + "packetPrinterFeed.wPaperPathStatus              = {3}\r\n"
                                                   + "packetPrinterFeed.wPaperPathState               = {4}\r\n"
                                                   , packetPrinterFeed.ubProtocolRevision.ToString()
                                                   , packetPrinterFeed.ubState.ToString()
                                                   , packetPrinterFeed.ubNrOfPaperPaths.ToString()
                                                   , "[" + packetPrinterFeed.wPaperPathStatus[0].ToString("X04") + "]/"
                                                   + "[" + packetPrinterFeed.wPaperPathStatus[1].ToString("X04") + "]/"
                                                   + "[" + packetPrinterFeed.wPaperPathStatus[2].ToString("X04") + "]"
                                                   , packetPrinterFeed.ubPaperPathState[0].ToString() + "/"
                                                   + packetPrinterFeed.ubPaperPathState[1].ToString() + "/"
                                                   + packetPrinterFeed.ubPaperPathState[2].ToString()
                                                   );
                    }
                }

                //-- DisconnectPrinter the printer
                disconnectPrinter(printerHandle);
            }

            //-- Output printer data to text box
            m_txtPrinterData.Text = printerData;
        }

        // -- Button clicked method for inquiring presenter status and output it to the printer data text box
        private void m_btnInquirePresenterStatus_Click(object sender, EventArgs e)
        {
            String printerData   = "<NO DATA>"; // String containing printer data
          
            // -- Connect to printer 
            IntPtr printerHandle = IntPtr.Zero; // Printer handle
            if ((printerHandle = connectPrinter()) != IntPtr.Zero)
            {
                // -- Clear data reception buffer buffer
                m_xpmApi.Xpm_Api_Pio_PrinterReadClear(printerHandle);

                // -- Request presenter status from printer
                ushort packetId       = Xpm_ApiWrapper.D_XPM_API_RID_RX_PRINTER_PRESENT;
                byte[] printerCommand = { 0x1d, 0x61, (byte) (packetId >> 8), (byte) (packetId & 0xFF)};
                if (m_xpmApi.Xpm_Api_Pio_PrinterWrite(printerHandle, printerCommand, (uint)printerCommand.Length, 100) == printerCommand.Length)
                {
                    // -- Process data transmitted by printer 
                    Xpm_ApiWrapper.TS_XPM_API_PCKT_HEADER            packetHeader          = new Xpm_ApiWrapper.TS_XPM_API_PCKT_HEADER();
                    Xpm_ApiWrapper.TS_XPM_API_PCKT_PRESENTER_STATUS  packetPresenterStatus = new Xpm_ApiWrapper.TS_XPM_API_PCKT_PRESENTER_STATUS();
                    if (m_xpmApi.Xpm_Api_Pio_PrinterRead(printerHandle, packetHeader, packetPresenterStatus, 500) < 0)
                    {
                        // -- No data received
                        printerData = String.Format("Printer data reception fault: No data received");
                    }
                    else
                    // .. Validate packet identifier 
                    if (packetHeader.ubId != Xpm_ApiWrapper.D_XPM_API_PID_TX_PRINTER_PRESENT)
                    {
                        // -- Invalid packet identifier
                        printerData = String.Format( "Printer data reception fault: Invalid packet identifier ({0} instead of {1})"
                                                   , packetHeader.ubId
                                                   , Xpm_ApiWrapper.D_XPM_API_PID_TX_PRINTER_PRESENT);
                    }
                    else
                    // .. Verify packet length
                    if (packetHeader.ubLength != Marshal.SizeOf(packetPresenterStatus))
                    {
                        // -- Invalid packet length
                        printerData = String.Format( "Data reception fault: Packet length mismatch ({0} instead of {1})"
                                                   , packetHeader.ubLength
                                                   , Marshal.SizeOf(packetPresenterStatus));
                    }   
                    // .. Process printer data
                    else
                    {    
                        //-- Process printer data
                        printerData = String.Format( "packetPresenterStatus.ubProtocolRevision        = {0}\r\n"
                                                   + "packetPresenterStatus.ubType                    = {1}\r\n"
                                                   + "packetPresenterStatus.wStatus                   = {2}\r\n"
                                                   + "packetPresenterStatus.ubNrOfLoadedPrintouts     = {3}\r\n"
                                                   + "packetPresenterStatus.dwNrOfLoadedPrintouts     = {4}\r\n"
                                                   + "packetPresenterStatus.dwNrOfTakenPrintouts      = {5}\r\n"
                                                   + "packetPresenterStatus.dwNrOfRejectedPrintouts   = {6}\r\n"
                                                   , packetPresenterStatus.ubProtocolRevision.ToString()
                                                   , packetPresenterStatus.ubType.ToString()
                                                   , "[" + packetPresenterStatus.wStatus.ToString("X04") + "]"
                                                   , packetPresenterStatus.ubNrOfLoadedPrintouts.ToString("000")
                                                   , packetPresenterStatus.dwNrOfLoadedPrintouts.ToString("00000")
                                                   , packetPresenterStatus.dwNrOfTakenPrintouts.ToString("00000")
                                                   , packetPresenterStatus.dwNrOfRejectedPrintouts.ToString("00000")
                                                   );
                    }
                }

                //-- DisconnectPrinter the printer
                disconnectPrinter(printerHandle);
            }

            //-- Output printer data to text box
            m_txtPrinterData.Text = printerData;
        }

        // -- Button clicked method for inquiring life cycle data and output it to the printer data text box
        private void m_btnInquireLifeCycle_Click(object sender, EventArgs e)
        {
            String printerData   = "<NO DATA>"; // String containing printer data
          
            // -- Connect to printer 
            IntPtr printerHandle = IntPtr.Zero; // Printer handle
            if ((printerHandle = connectPrinter()) != IntPtr.Zero)
            {
                // -- Clear data reception buffer buffer
                m_xpmApi.Xpm_Api_Pio_PrinterReadClear(printerHandle);

                // -- Request life cycle data from printer
                ushort packetId       = Xpm_ApiWrapper.D_XPM_API_RID_RX_PRINTER_LIFE;
                byte[] printerCommand = { 0x1d, 0x61, (byte) (packetId >> 8), (byte) (packetId & 0xFF)};
                if (m_xpmApi.Xpm_Api_Pio_PrinterWrite(printerHandle, printerCommand, (uint)printerCommand.Length, 100) == printerCommand.Length)
                {
                    // -- Process data transmitted by printer 
                    Xpm_ApiWrapper.TS_XPM_API_PCKT_HEADER              packetHeader    = new Xpm_ApiWrapper.TS_XPM_API_PCKT_HEADER();
                    Xpm_ApiWrapper.TS_XPM_API_PCKT_PRINTER_LIFE_CYCLE  packetLifeCycle = new Xpm_ApiWrapper.TS_XPM_API_PCKT_PRINTER_LIFE_CYCLE();
                    if (m_xpmApi.Xpm_Api_Pio_PrinterRead(printerHandle, packetHeader, packetLifeCycle, 500) < 0)
                    {
                        // -- No data received
                        printerData = String.Format("Printer data reception fault: No data received");
                    }
                    else
                    // .. Validate packet identifier 
                    if (packetHeader.ubId != Xpm_ApiWrapper.D_XPM_API_PID_TX_PRINTER_LIFE)
                    {
                        // -- Invalid packet identifier
                        printerData = String.Format( "Printer data reception fault: Invalid packet identifier ({0} instead of {1})"
                                                   , packetHeader.ubId
                                                   , Xpm_ApiWrapper.D_XPM_API_PID_TX_PRINTER_LIFE);
                    }
                    else
                    // .. Verify packet length
                    if (packetHeader.ubLength != Marshal.SizeOf(packetLifeCycle))
                    {
                        // -- Invalid packet length
                        printerData = String.Format( "Data reception fault: Packet length mismatch ({0} instead of {1})"
                                                   , packetHeader.ubLength
                                                   , Marshal.SizeOf(packetLifeCycle));
                    }   
                    // .. Process printer data
                    else
                    {    
                        //-- Process printer data
                        printerData = String.Format( "packetLifeCycle.ubProtocolRevision              = {0}\r\n"
                                                   + "packetLifeCycle.dwTimePoweredS                  = {1}\r\n"
                                                   + "packetLifeCycle.dwDistanceSteppedMm             = {2}\r\n"
                                                   + "packetLifeCycle.dwDistanceBurnedMm              = {3}\r\n"
                                                   + "packetLifeCycle.dwPaperTransportedMm            = {4}\r\n"
                                                   + "packetLifeCycle.dwNrOfSystemStarts              = {5}\r\n"
                                                   + "packetLifeCycle.dwNrOfHardResets                = {6}\r\n"
                                                   + "packetLifeCycle.dwNrOfPaperCuts                 = {7}\r\n"
                                                   + "packetLifeCycle.wNrOfPaperLoads                 = {8}\r\n"
                                                   , packetLifeCycle.ubProtocolRevision.ToString()
                                                   , packetLifeCycle.dwTimePoweredS.ToString("0000000000") + " s"
                                                   , packetLifeCycle.dwDistanceSteppedMm.ToString("0000000000")  + " mm"
                                                   , packetLifeCycle.dwDistanceBurnedMm.ToString("0000000000")  + " mm"
                                                   , packetLifeCycle.dwPaperTransportedMm.ToString("0000000000")  + " mm"
                                                   , packetLifeCycle.dwNrOfSystemStarts.ToString("0000000000") 
                                                   , packetLifeCycle.dwNrOfHardResets.ToString("0000000000")
                                                   , packetLifeCycle.dwNrOfPaperCuts.ToString("0000000000")
                                                   , packetLifeCycle.wNrOfPaperLoads.ToString("0000000000")
                                                   , packetLifeCycle.dwNrOfPaperCuts.ToString("0000000000")
                                                   );
                    }
                }

                //-- DisconnectPrinter the printer
                disconnectPrinter(printerHandle);
            }

            //-- Output printer data to text box
            m_txtPrinterData.Text = printerData;
        }

        // -- Button clicked method for inquiring ticket data and output it to the printer data text box
        private void m_btnInquirePrinterTicket_Click(object sender, EventArgs e)
        {
            String printerData   = "<NO DATA>"; // String containing printer data
            System.Windows.Forms.Button Button = (System.Windows.Forms.Button) sender;

            Button.UseWaitCursor = true;

            // -- Connect to printer 
            IntPtr printerHandle = IntPtr.Zero; // Printer handle
            if ((printerHandle = connectPrinter()) != IntPtr.Zero)
            {
                // -- Clear data reception buffer buffer
                m_xpmApi.Xpm_Api_Pio_PrinterReadClear(printerHandle);

                // -- Request ticket data from printer
                ushort packetId       = Xpm_ApiWrapper.D_XPM_API_RID_RX_PRINTER_TICKET;
                byte[] printerCommand = { 0x1d, 0x61, (byte) (packetId >> 8), (byte) (packetId & 0xFF)};
                if (m_xpmApi.Xpm_Api_Pio_PrinterWrite(printerHandle, printerCommand, (uint)printerCommand.Length, 100) == printerCommand.Length)
                {
                    // -- Process data transmitted by printer 
                    Xpm_ApiWrapper.TS_XPM_API_PCKT_HEADER             packetHeader        = new Xpm_ApiWrapper.TS_XPM_API_PCKT_HEADER();
                    Xpm_ApiWrapper.TS_XPM_API_PCKT_PRINTER_TICKET     packetPrinterTicket = new Xpm_ApiWrapper.TS_XPM_API_PCKT_PRINTER_TICKET();
                    if (m_xpmApi.Xpm_Api_Pio_PrinterRead(printerHandle, packetHeader, packetPrinterTicket, 500) < 0)
                    {
                        // -- No data received
                        printerData = String.Format("Printer data reception fault: No data received");
                    }
                    else
                    // .. Validate packet identifier 
                    if (packetHeader.ubId != Xpm_ApiWrapper.D_XPM_API_PID_TX_PRINTER_TICKET)
                    {
                        // -- Invalid packet identifier
                        printerData = String.Format( "Printer data reception fault: Invalid packet identifier ({0} instead of {1})"
                                                   , packetHeader.ubId
                                                   , Xpm_ApiWrapper.D_XPM_API_PID_TX_PRINTER_TICKET);
                    }
                    else
                    // .. Verify packet length
                    if (packetHeader.ubLength != Marshal.SizeOf(packetPrinterTicket))
                    {
                        // -- Invalid packet length
                        printerData = String.Format( "Data reception fault: Packet length mismatch ({0} instead of {1})"
                                                   , packetHeader.ubLength
                                                   , Marshal.SizeOf(packetPrinterTicket));
                    }   
                    // .. Process printer data
                    else
                    {    
                        //-- Process printer data
                        printerData = String.Format( "packetPrinterTicket.ubProtocolRevision          = {0}\r\n"
                                                   + "packetPrinterTicket.ubTicketId                  = {1}\r\n"
                                                   + "packetPrinterTicket.wTicketLengthMm             = {2}\r\n"
                                                   + "packetPrinterTicket.wPrintedLengthMm            = {3}\r\n"
                                                   + "packetPrinterTicket.ubPrintedAreaMinPrcnt       = {4}\r\n"
                                                   + "packetPrinterTicket.ubPrintedAreaMaxPrcnt       = {5}\r\n"
                                                   + "packetPrinterTicket.wTicketStatus               = {6}\r\n"
                                                   , packetPrinterTicket.ubProtocolRevision.ToString()
                                                   , packetPrinterTicket.ubTicketId.ToString("000") 
                                                   , packetPrinterTicket.wTicketLengthMm.ToString("00000")  + " mm"
                                                   , packetPrinterTicket.wPrintedLengthMm.ToString("00000")  + " mm"
                                                   , packetPrinterTicket.ubPrintedAreaMinPrcnt.ToString("000")  + " %"
                                                   , packetPrinterTicket.ubPrintedAreaMaxPrcnt.ToString("000")  + " %"
                                                   , "[" + packetPrinterTicket.wTicketStatus.ToString("X04") + "]"
                                                   );
                    }
                }

                //-- DisconnectPrinter the printer
                disconnectPrinter(printerHandle);
            }

            //-- Output printer data to text box
            m_txtPrinterData.Text = printerData;

            Button.UseWaitCursor = false;
        }        

        // -- Button for printing data received from printer
        private void m_btnPrintData_Click(object sender, EventArgs e)
        {
            String printerData   = "<NO DATA>"; // String containing printer data
          
            //-- Output printer data to text box
            if (m_txtPrinterData.Text.Count() > 0)
               printerData = m_txtPrinterData.Text;

            // -- Connect to printer 
            IntPtr printerHandle = IntPtr.Zero; // Printer handle
            if ((printerHandle = connectPrinter()) != IntPtr.Zero)
            {
                // -- Create print buffer
                char [] printBuffer = new char[0]; 

                // -- Add soft reset the print buffer
                byte[] cmdSoftReset=  { 0x1B, 0x40 };
                Array.Resize(ref printBuffer, printBuffer.Length + cmdSoftReset.Length);
                cmdSoftReset.CopyTo(printBuffer, printBuffer.Length - cmdSoftReset.Length);
               
                // -- Add printer data to print buffer
                Array.Resize(ref printBuffer, printBuffer.Length + printerData.Length);
                Encoding.ASCII.GetBytes(printerData).CopyTo(printBuffer, printBuffer.Length - printerData.Length);

                // -- Add form feed to print buffer
                byte[] cmdFormFeed =  { 0x0C };
                Array.Resize(ref printBuffer, printBuffer.Length + cmdFormFeed.Length);
                cmdFormFeed.CopyTo(printBuffer, printBuffer.Length - cmdFormFeed.Length);

                // -- Add full cut command to print buffer
                byte[] cmdFullCut =  { 0x1B, 0xF0, 0x06, 0x01, 0x02 };
                Array.Resize(ref printBuffer, printBuffer.Length + cmdFullCut.Length);
                cmdFullCut.CopyTo(printBuffer, printBuffer.Length - cmdFullCut.Length);

                //-- Write the print buffer
                m_xpmApi.Xpm_Api_Pio_PrinterWrite(printerHandle, printBuffer, (uint) printBuffer.Length, 5000);

                //-- DisconnectPrinter the printer
                disconnectPrinter(printerHandle);
            }
        }

        // -- Print image on printer
        private void m_btnPrintImage_Click(object sender, EventArgs e)
        {
 
            // -- Show open file dialog for 
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = "Image file (*.jpg;*.bmp;*.png;*.gif;*.tiff;*.tif)|*.jpg;*.bmp;*.png;*.gif;*.tiff;*.tif|All files (*.*)|*.*";
            if (openFileDialog.ShowDialog() == DialogResult.OK) {
     
                // -- Connect to printer 
                IntPtr printerHandle = IntPtr.Zero; // Printer handle
                if ((printerHandle = connectPrinter()) != IntPtr.Zero)
                { 
                    // -- Print image on printer
                    m_xpmApi.Xpm_Api_Pio_PrintImage(printerHandle, openFileDialog.FileName, 5000);

                    // -- Send full cut command to print buffer
                    byte[] cmdFullCut =  { 0x1B, 0xF0, 0x06, 0x01, 0x02 };
                    m_xpmApi.Xpm_Api_Pio_PrinterWrite(printerHandle, cmdFullCut, (uint) cmdFullCut.Length, 5000);

                    //-- DisconnectPrinter the printer
                    disconnectPrinter(printerHandle); 
                }
            }
        }

        // -- Exit button 
        private void m_btnExit_Click(object sender, EventArgs e)
        {
            // -- Close form
            Close();
        }      
    }
}
