/***********************************************************************\
**
**                          Copyright (c) 2008-2022 by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a license and may be used and
**      copied only in accordance with the terms  of  such  license  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Xpm_Pmp_LogMan.cpp
**  Facility:    S690001 - Hengstler XPM Printer Driver for Windows XP/7/8/10
**  Component:   Print monitor (Xpm-PrintMon)
**  Version:     R8-V1.07-FINAL / 2022OCT18
**  Environment: Windows XP/7/8/10
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Module that manages the info entry in the registry
**  Note:        -
**  History:     R1-V1.00c / 2008JUL11 / HOR / 1st RELEASE - 1st BETA
**               R1-V1.00d / 2008DEC11 / HOR / 1st RELEASE - 2nd BETA
**               R1-V1.00e / 2009MAY05 / HOR / 1st RELEASE - 3rd BETA
**               R1-V1.00f / 2009JUL16 / HOR / 1st RELEASE - 4th BETA
**               R1-V1.00  / 2009AUG27 / HOR / 1st RELEASE - FINAL 
**               ------------------------------------------------------------
**               R2-V1.01a / 2010JAN26 / HOR / 2nd RELEASE - 1st BETA 
**               R2-V1.01b / 2010MAR04 / HOR / 2nd RELEASE - 2nd BETA
**               R2-V1.01c / 2011DEC19 / HOR / M-R2-012-HOR
**                  Complete driver package converted to VS2010 and adapted to Windows 7 32/64 bit
**               R2-V1.01c / 2011DEC19 / HOR / 2nd RELEASE - 3rd BETA
**               R2-V1.01d / 2012JUN27 / HOR / 2nd RELEASE - 4th BETA
**               R2-V1.01e / 2015MAY05 / HOR / 2nd RELEASE - 5th BETA
**               R2-V1.01f / 2016FEB15 / HOR / 2nd RELEASE - 6th BETA
**               R2-V1.01  / 2016AUG19 / HOR / M-R2-041-HOR
**                 Complete driver package converted to VS2015 and adapted to Windows 8/10 32/64 bit
**               R2-V1.01  / 2016AUG19 / HOR / 2nd RELEASE - FINAL
**               ------------------------------------------------------------
**               R3-V1.02  / 2016AUG24 / HOR / M-R3-001-HOR
**                 Additional driver package compilation added for usage within Windows-XP.
**               R3-V1.02  / 2016AUG24 / HOR / 3rd RELEASE - FINAL
**               ------------------------------------------------------------
**               R4-V1.03  / 2018JUL10 / HOR / 4th RELEASE - FINAL
**               ------------------------------------------------------------
**               R5-V1.04  / 2018OCT11 / HOR / 5th RELEASE - FINAL
**               ------------------------------------------------------------
**               R6-V1.05  / 2018DEC07 / HOR / 6th RELEASE - FINAL
**               ------------------------------------------------------------
**               R7-V1.06  / 2022FEB10 / HOR / 7th RELEASE - FINAL
**               ------------------------------------------------------------
**               R8-V1.07  / 2022OCT18 / HOR / 8th RELEASE - FINAL
**               ------------------------------------------------------------
**
\**********************************************************************/

/*
** INCLUDES
*/

#include "Xpm_Pmp_System.h"
#include <shlwapi.h>
#include "Xpm_Pmp_LogMan.h"

/*
** COMPILER 
*/

/*
** MACROS
*/

/*
** TYPES
*/

/*
** FUNCTIONS
*/

/*
** VARIABLES
*/

//******************************************************************************************
// METHOD:   CXpm_Pmp_LogMan::CXpm_Pmp_LogMan
// ABSTRACT: Object constructor
//******************************************************************************************

CXpm_Pmp_LogMan::CXpm_Pmp_LogMan
   ()
{
   //-- Initialize the mutex for exclusive access
   InitializeCriticalSection(&m_csAccess);

   //-- Initialize variables
   m_szLogFile[0] = '\0';
   m_pfLogFile    = NULL;
   
   //-- Open log file
   //.. this->OpenLogFile();
}

//******************************************************************************************
// METHOD:   CXpm_Pmp_LogMan::~CXpm_Pmp_IoMan
// ABSTRACT: Object destructor
//******************************************************************************************

CXpm_Pmp_LogMan::~CXpm_Pmp_LogMan
   ()
{
   //-- Close log file
   this->CloseLogFile();
}

//******************************************************************************************
// METHOD:   CXpm_Pmp_LogMan::OpenLogFile
// ABSTRACT: Opens the log file (if enabled)
//******************************************************************************************

void CXpm_Pmp_LogMan::OpenLogFile
   (void)
{
   char szDrive[_MAX_DRIVE];    // Drive id
   char szDir[_MAX_DIR];        // Directory 
   char szFname[_MAX_FNAME];    // Filename 
   char szExt[_MAX_EXT];        // Extension 
     
   //-- Close log file if open
   if (m_pfLogFile) fclose(m_pfLogFile); m_pfLogFile = NULL;

   //-- Use path to this module 
   GetModuleFileNameA(NULL, m_szLogFile, sizeof(m_szLogFile));

   //-- Split path 
   _splitpath(m_szLogFile, szDrive, szDir, szFname, szExt);

   //-- Create path to directory
   _makepath(m_szLogFile, szDrive, szDir, NULL, NULL);

   //-- Add name of module
   strcat(m_szLogFile, "Xpm_Pmp");
     
   //-- Add date time
   {
      //-- Obtain local time 
      GetLocalTime(&m_sLogFileSysTime);

      //-- Print time
      sprintf(&m_szLogFile[strlen(m_szLogFile)], "_%4.4d%2.2d%2.2d_%2.2d%2.2d%2.2d"
             , m_sLogFileSysTime.wYear, m_sLogFileSysTime.wMonth, m_sLogFileSysTime.wDay
             , m_sLogFileSysTime.wHour, m_sLogFileSysTime.wMinute, m_sLogFileSysTime.wSecond 
           );
   }

   //-- Add extension
   strcat(m_szLogFile, ".log");

   //-- Open log file
   if ((m_pfLogFile = fopen(m_szLogFile, "w+")) != NULL) {

      //-- Print info on log file
      this->LogPrint("LOG-MAN", "LOG-FILE %s", m_szLogFile);
   }
}

//******************************************************************************************
// METHOD:   CXpm_Pmp_LogMan::CloseLog
// ABSTRACT: Closes the log file (if enabled)
//******************************************************************************************

void CXpm_Pmp_LogMan::CloseLogFile
   (void)
{
   //-- Close log file if open
   if (m_pfLogFile) fclose(m_pfLogFile); m_pfLogFile = NULL;
}

//******************************************************************************************
// METHOD:   CXpm_Pmp_LogMan::ManageLogFile
// ABSTRACT: Manages the log file
//******************************************************************************************

void CXpm_Pmp_LogMan::ManageLogFile
   (void)
{
   //-- Check if log file is open
   if (m_pfLogFile) {

      SYSTEMTIME sSysTime; // System time

      //-- Obtain local time 
      GetLocalTime(&sSysTime);

      //-- Start new log if day changes
      if (    (sSysTime.wYear  != m_sLogFileSysTime.wYear)
           || (sSysTime.wMonth != m_sLogFileSysTime.wMonth)
           || (sSysTime.wDay   != m_sLogFileSysTime.wDay)
         ) {

         //-- Close log file
         this->CloseLogFile();

         //-- Open log file
         this->OpenLogFile();
      }
   }
}

//******************************************************************************************
// METHOD:   CXpm_Pmp_LogMan::LogPrint
// ABSTRACT: Prints to the log file
//******************************************************************************************

void CXpm_Pmp_LogMan::LogPrint
   (PSTR pszInfo, PSTR pszFormat, ...) 
{
   va_list pVaArg;       // Variable arguments pointer
   CHAR    szBuff[1000]; // A buffer

   //-- Check if log file is opened
   if (!m_pfLogFile)
      goto EXIT;

   //-- Manage the log file
   this->ManageLogFile();

   //-- Write greenwich mean time
   {
      SYSTEMTIME sSysTime;
   
      //-- Obtain local time
      GetLocalTime(&sSysTime);

      //-- Print time
      sprintf(szBuff, "%4.4d.%2.2d.%2.2d %2.2d:%2.2d:%2.2d.%3.3d"
             , sSysTime.wYear, sSysTime.wMonth, sSysTime.wDay
             , sSysTime.wHour, sSysTime.wMinute, sSysTime.wSecond , sSysTime.wMilliseconds 
             );

      //-- Write time information 
      fprintf(m_pfLogFile, "%s ", szBuff);
   }

   //-- Print log info
   {
      //-- Write time information 
      fprintf(m_pfLogFile, "%-10.10s ", pszInfo ? pszInfo : "");
   }

   //-- Process variable arguments
   {
      //-- Initialize variable arguments, write to file and reset variable arguments
      va_start(pVaArg, pszFormat);
      vfprintf(m_pfLogFile, pszFormat, pVaArg);
      va_end(pVaArg);
   }

   //-- Terminate line
   fprintf(m_pfLogFile, "\n");

   //-- Flush log file
   fflush(m_pfLogFile);

   //-- All done
   goto EXIT;

EXIT:

   ;
}   

//******************************************************************************************
// METHOD:   CXpm_Pmp_LogMan::LogData
// ABSTRACT: Prints data to the log file
//******************************************************************************************

void CXpm_Pmp_LogMan::LogData
   (PBYTE pubData, INT nData, char cFlag) 
{
   bool  fTrucated = false; // Flag indicicating log data trucated
   INT   nBytes;            // Number of bytes, 
   INT   iData;             // Index in log data
   DWORD dwLogDataMax;      // Maximum number of bytes to log

   //-- Check if log file is opened
   if (!m_pfLogFile)
      goto EXIT;

   //-- Check if data is to be logged
   if (!nData)
      goto EXIT;

   //-- Log all data
   dwLogDataMax = nData;

   //-- Determine number of bytes, limit to maximum
   if ((nBytes = nData) > (INT) dwLogDataMax) {
      nBytes  = (INT) dwLogDataMax / 2;
      fTrucated = true; 
   }

   //-- Set offset of first part 
   iData = 0;

   //-- Process all bytes
   for (; nBytes > 0; iData += 32, nBytes -= 32) {
      
      //-- Process up to n bytes
      fprintf(m_pfLogFile, "%c 0x%8.8x [", cFlag ? cFlag : '-', iData);
      for (int iCnt = 0; iCnt < 32; iCnt++) {
         if (iCnt)
            fprintf(m_pfLogFile, " ");
         if (iData + iCnt < nData) 
              fprintf(m_pfLogFile, "%2.2X", pubData[iData + iCnt]);
         else fprintf(m_pfLogFile, "--");
      }
      fprintf(m_pfLogFile, "] \""); 
      for (int iCnt = 0; iCnt < 32; iCnt++) {
         if (iData + iCnt < nData) 
              fprintf(m_pfLogFile, "%c", isprint(pubData[iData + iCnt]) ? pubData[iData + iCnt] : '' );
         else 
         if (nBytes > 32)
            fprintf(m_pfLogFile, "-");
      }
      fprintf(m_pfLogFile, "\"\n"); 
   }

   //-- Inform if trucated 
   if (fTrucated) {

      //-- Show break of data 
      fprintf(m_pfLogFile, ">  DATA TRUNCATED TO %d KB => ONLY FIRST AND LAST PART DISPLAYED\n", dwLogDataMax / 1024);

      //-- Determine offset of second part 
      iData = ((nData - dwLogDataMax + 31) / 32) * 32;

      //-- Determine number of bytes to maximum
      nBytes  = nData - iData;

      //-- Process all bytes
      for (; nBytes > 0; iData += 32, nBytes -= 32) {
      
         //-- Process up to n bytes
         fprintf(m_pfLogFile, "> 0x%8.8x [", iData);
         for (int iCnt = 0; iCnt < 32; iCnt++) {
            if (iCnt)
               fprintf(m_pfLogFile, " ");
            if (iData + iCnt < nData) 
                 fprintf(m_pfLogFile, "%2.2X", pubData[iData + iCnt]);
            else fprintf(m_pfLogFile, "--");
         } 
         fprintf(m_pfLogFile, "] \""); 
         for (int iCnt = 0; iCnt < 32; iCnt++) {
            if (iData + iCnt < nData) 
                 fprintf(m_pfLogFile, "%c", isprint(pubData[iData + iCnt]) ? pubData[iData + iCnt] : '' );
            else 
            if (nBytes > 32)
               fprintf(m_pfLogFile, "-");
         }
         fprintf(m_pfLogFile, "\"\n"); 
      }
   }

   //-- Flush log file
   fflush(m_pfLogFile);

   //-- All done
   goto EXIT;

EXIT:

   ;
}   
